/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.field.shape;

import nl.cloudfarming.client.field.model.Shape;
import com.vividsolutions.jts.geom.Geometry;
import java.awt.datatransfer.Transferable;
import java.io.IOException;
import java.util.List;
import javax.swing.Action;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.geotools.data.FeatureSource;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.opengis.feature.simple.SimpleFeature;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.*;
import org.openide.util.Exceptions;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

public class ShapeDataObject extends MultiDataObject {

    public static final String MIMETYPE = "application/x-esri-shape";

    public ShapeDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor(MIMETYPE, true);
    }

    @Override
    protected Node createNodeDelegate() {
        Node delegate = super.createNodeDelegate();
        return new ShapeDataObjectNode(delegate, getPrimaryFile().getName());
    }

    /**
     * Root node for the shapefile
     */
    private final class ShapeDataObjectNode extends AbstractNode {

        final Node origional;

        public ShapeDataObjectNode(Node origional, String name) {
            super(Children.create(new ShapeChildFactory(), true),new ProxyLookup(origional.getLookup(),Lookups.singleton(new SimpleLayer(name))));
            this.origional = origional;
            setIconBaseWithExtension("nl/cloudfarming/client/field/shape/file_extension_shp.png");
            setDisplayName(name);
        }

        @Override
        public Action[] getActions(boolean context) {
            return origional.getActions(context);
        }
    }
    
    /**
     * Node representing a SimpleFeature (shape) in a shapefile
     */
    private final class ShapeNode extends AbstractNode{
        private final Shape shape;

        public ShapeNode(Shape shape) {
            super(Children.LEAF, Lookups.fixed(shape, new SimpleLayer(shape.getName())));
            this.shape = shape;
            setName(shape.getName());
            setIconBaseWithExtension("nl/cloudfarming/client/field/shape/file_extension_shp.png");
        }
       
        @Override
        public Transferable drag() throws IOException {
            return clipboardCopy();
        }

        @Override
        public Transferable clipboardCopy() throws IOException {
            try {
                return (Shape) shape.clone();
            } catch (CloneNotSupportedException ex) {
                Exceptions.printStackTrace(ex);
                return null;
            }
        }        
    }

    /**
     * Creates shape nodes for the simple features within the specified shapfile
     */
    private final class ShapeChildFactory extends ChildFactory<SimpleFeature> {

        @Override
        protected boolean createKeys(List<SimpleFeature> toPopulate) {
            try {
                FileObject fileObject = ShapeDataObject.this.getPrimaryFile();

                ShapefileDataStore shpDataStore = new ShapefileDataStore(fileObject.getURL());
                String typeName = shpDataStore.getTypeNames()[0];
                FeatureSource featureSource = shpDataStore.getFeatureSource(typeName);

                FeatureCollection result = featureSource.getFeatures();
                FeatureIterator iterator = result.features();
                while (iterator.hasNext()) {
                    SimpleFeature feature = (SimpleFeature) iterator.next();
                    toPopulate.add(feature);
                }
                iterator.close();
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
            return true;
        }

        @Override
        protected Node createNodeForKey(SimpleFeature key) {
            Shape shape = parseShape(key);
            Node shapeNode = new ShapeNode(shape);
            return shapeNode;
        }

        /**
         *
         * @param payload
         * @return
         */
        private Shape parseShape(SimpleFeature payload) {

            int attIndexPlotPoly = payload.getFeatureType().indexOf("the_geom");
            int attIndexPlotName = payload.getFeatureType().indexOf("KAVEL");
            Geometry geometry = null;
            if (attIndexPlotPoly >= 0) {
                geometry = (Geometry) payload.getAttribute(attIndexPlotPoly);
            }
            String shapeName = "Unknown";
            if (attIndexPlotName >= 0) {
                shapeName = (String) payload.getAttribute(attIndexPlotName);
            }

            Shape shape = new Shape(geometry);
            shape.setName(shapeName);
            return shape;
        }
    }

    @Override
    protected int associateLookup() {
        return 1;
    }
//    @MultiViewElement.Registration(displayName = "#LBL_Shape_EDITOR",
//    iconBase = "nl/cloudfarming/client/field/shape/file_extension_shp.png",
//    mimeType = MIMETYPE,
//    persistenceType = TopComponent.PERSISTENCE_ONLY_OPENED,
//    preferredID = "Shape",
//    position = 1000)
//    @Messages("LBL_Shape_EDITOR=Source")
//    public static MultiViewEditorElement createEditor(Lookup lkp) {
//        return new MultiViewEditorElement(lkp);
//    }
}
